# !/usr/bin/env python3
# -*- coding: UTF-8 -*-

"""The ``main`` module represents the main django application. It handles the
settings and URL routing for the website and includes minimal custom views or
models.
"""

from . import asgi, settings, urls, views, wsgi
