# !/usr/bin/env python3
# -*- coding: UTF-8 -*-

"""Custom django applications built for the Pitt-Google Broker website"""

# Todo apps:
# - The ``alerts`` and ``objects`` apps are very similar, but I expect them
#   to diverge during development. If this is incorrect, consider merging them.
