# !/usr/bin/env python3
# -*- coding: UTF-8 -*-

"""A django application for managing user profiles and topic subscriptions"""

# Todo-subscriptions:
# - Add a model to which users can manage their topic subscriptions
