# !/usr/bin/env python3
# -*- coding: UTF-8 -*-

"""Adds commands to the default Django management script ``manage.py``

For more information see:
  https://docs.djangoproject.com/en/3.0/howto/custom-management-commands/
"""
